/**
 * A class providing highway functionality
 * @author Megan Englert
 */


import java.awt.*;
import java.lang.*;

public class Highway extends Street {





    private TravQueue[][] driving = new TravQueue[2][2];
    private TravQueue[][] waiting = new TravQueue[2][2];

    public Highway(String nm, int lim) {
	super(nm, lim);
	width *= 2;
	driving[0][0] = new TravQueue();
	driving[0][1] = new TravQueue();
	driving[1][0] = new TravQueue();
	driving[1][1] = new TravQueue();
	waiting[0][0] = new TravQueue();
	waiting[0][1] = new TravQueue();
	waiting[1][0] = new TravQueue();
	waiting[1][1] = new TravQueue();
	System.out.println("Highway");
    }


  /**
   * draw the street
   * @param g the graphics environment
   */
  public void draw(Graphics g) {
      super.draw(g);

    driving[0][0].draw(g);
    driving[0][1].draw(g);
    driving[1][0].draw(g);
    driving[1][1].draw(g);
    waiting[0][0].draw(g);
    waiting[0][1].draw(g);
    waiting[1][0].draw(g);
    waiting[1][1].draw(g);

  }

  /**
   * determine if a street waiting list is empty
   * @param d the direction (0 or 1)
   * @return true or false
   */
  public boolean isEmpty( int d ) {
    boolean res = waiting[d][0].isEmpty();
    Vehicle v = null;

    if( !res ) {
      try {
	v = (Vehicle) waiting[d][0].peek();
      } catch (Exception e) {
	System.out.println("Exception: " + e.getMessage() );
	Global.debug.println("Exception: " + e.getMessage());
      }
        if(Global.DEBUG) {
          Global.debug.println("Vehicle front check");
          Global.debug.println(" - street loc[" + d + "] is " + location[d]);
          Global.debug.println(" - vehicle loc is " + v.where );
          Global.debug.println(" - vehicle distance is " 
                               + v.where.distance(location[d]) );
       }
      if(limit > 50) {
	res = (v.where.distance(location[d]) > (4*Global.AT_FRONT) );
      } else {
	res = (v.where.distance(location[d]) > Global.AT_FRONT );
      }
    }
    return res;
  }

  public boolean isHighwayEmpty( int d ) {
    boolean res = waiting[d][1].isEmpty();
    Vehicle v = null;

    if( !res ) {
      try {
	v = (Vehicle) waiting[d][1].peek();
      } catch (Exception e) {
	System.out.println("Exception: " + e.getMessage() );
	Global.debug.println("Exception: " + e.getMessage());
      }
        if(Global.DEBUG) {
          Global.debug.println("Vehicle front check");
          Global.debug.println(" - street loc[" + d + "] is " + location[d]);
          Global.debug.println(" - vehicle loc is " + v.where );
          Global.debug.println(" - vehicle distance is " 
                               + v.where.distance(location[d]) );
       }

	res = (v.where.distance(location[d]) > (6*Global.AT_FRONT) );

    }
    return res;
  }

    /**
     * dequeue from the street waiting list
     * @param d the direction (0 or 1)
     * @param o the object to enqueue
     */
    public Vehicle dequeue( int d ) {

	return (Vehicle) waiting[d][0].dequeue();
    }

    public Vehicle highwayDequeue(int d) {
	return (Vehicle) waiting[d][1].dequeue();
    }


    /**
     * peek into the street waiting list
     * @param d the direction (0 or 1)
     * @param o the object to look at
     */
    public Vehicle peek( int d ) {

	return (Vehicle) waiting[d][0].peek();
    }

    public void enqueue(Vehicle v, int d) {
	Coord delta; 

	delta = direction[d].normal().scale( width/(5*Global.SCALE) );
	
	driving[d][0].enqueue(v);
	v.locate( location[1-d].diff(delta), direction[d] );
	v.update( v.getLength() + Global.STREET_WIDTH/Global.SCALE );

    }

    public void enterHighway(Vehicle v, int d) {
	Coord delta; 
	if(d == 0){
	    enqueue(v,d);
	}else {
	delta= direction[d].normal().scale( width/(2*Global.SCALE) );
	
	driving[d][1].enqueue(v);
	v.locate( location[1-d].diff(delta), direction[d] );
	v.update( v.getLength() + Global.STREET_WIDTH/Global.SCALE );
	}
    }
 /**
   * update the street
   * @param dt the time difference in seconds
   */
    public void update(double dt) {
	int i;

	for(i = 0; i < 2; i++ ) {// move all the vehicles
	    for(int j = 0; j < 2; j++) {
		waiting[i][j].update(dt * limit / 2);
		driving[i][j].update(dt * limit);
	    }
	}
	for(i = 0; i < 2; i++ ) {
	    for (int j = 0; j < 2; j++) {
		if(Global.DEBUG)
		    Global.debug.println("Street.update " + i);
		try {
		    qBack[i] = separate( waiting[i][j],
					 driving[i][j],
					 direction[1-i],
					 location[i].sum( direction[1-i].normal().scale( width/(2*Global.SCALE) )));
		} catch( Exception e) { 
		    System.out.print("Error " + e + " updating a Street" ); 
		    Global.debug.println("Error " + e + " updating a Street" ); 
		} 
	    }
	}
    }

  /**
   * document the contents of the street
   * @return the string describing the street
   */
  public void display() {
    int i;

    for( i = 0; i < 2; i++ ) {
      Global.debug.println(toString() + "[" + i + "]");
      Global.debug.println("location:" + location[i]
			   + "; direction:" + direction[i]
			   + "; qBack:" + qBack[i] );
      Global.debug.println("driving left hand lane:" + driving[i][1] );
      Global.debug.println("waiting left hand lane:" + waiting[i][1] );
      Global.debug.println("driving right hand lane:" + driving[i][0] );
      Global.debug.println("waiting right hand lane:" + waiting[i][0] );
    }
  }

}
