/**
 * A class providing highway intersection functionality
 * @author Megan Englert
 */

import java.awt.*;

public class HighwayIntersection extends Intersection {

    public HighwayIntersection(int[] stIndex, int[] dirIndex, Coord p ) {
	super(stIndex, dirIndex, p);
	System.out.println("Highway Intersection");
    }

  /**
   * draw the intersection
   * @param g the graphics environment
   */
  public void draw(Graphics g) {
    Coord here = position.screenScale();
    g.setColor(new Color(120, 120, 180) );
    g.fillOval( (int) (here.x - 4*Global.STREET_WIDTH),
		(int) (here.y - 4*Global.STREET_WIDTH),
		(int) (8 * Global.STREET_WIDTH),
		(int) (8 * Global.STREET_WIDTH) );
  }

    public void update(double x) {
	int index;
	Street strt;
	int direction;
	Street outStreet;
	int outDir;
	int i;
	int which;
	int choice;
	Vehicle v;
	/*
	  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	  ;;
	  ;;  this is the basic free flowing traffic control logic
	  ;;  it works fine as-is.  For other flow control methods,
	  ;;  derive a child class from this and change the logic accordingly
	  ;; 
	  ;;  4 way stop might cycle through the streets, taking only the 
	  ;;    first available car from one street
	  ;;
	  ;;  traffic lights might permit cars from odd or even indexed streets 
	  ;;    switching to the other state after some period of time
	*/
	for( i = 0; i < streetIndex.length; i++ ) {
	    index = streetIndex[i];
	    strt = (Street) Global.streets.find(index);
	    direction = directionIndex[i];
	    //if(Global.DEBUG)
	    //    Global.debug.print("Looked at <" + strt + ">");
	    if( !strt.isEmpty(direction) )  {
		if(Global.DEBUG)
		    Global.debug.println("Street <" + strt + " not empty");
		v = strt.peek(direction);
		// pick the outgoing street
		v.turn(this, strt, direction);
		outStreet = v.getNextStreet();
		outDir = v.getNextDirection();
		if( outStreet == null ) {
		    strt.dequeue(direction);
		} else {
		    if( !outStreet.isFull(outDir, v) ) {
			try {
			    outStreet.enqueue( strt.dequeue(direction), outDir );
			} catch( Exception e) { 
			    System.out.print("Error " + e + " dequeuing" ); 
			    Global.debug.println("Error " + e + " dequeuing" );
			} 
		    }
		}
	    }

	    if(strt instanceof Highway) {
		Highway next = null;

		if(streetIndex.length != 1) {
		    next = (Highway)Global.streets.find(streetIndex[streetIndex.length - 1]);
		    if(strt == next) {
			next = (Highway)Global.streets.find(streetIndex[1]);
		    }
		}


// 		for(int j = 0; j < streetIndex.length; j++) {
// 		    int dex = streetIndex[j];
// 		    Street st = (Street) Global.streets.find(dex);
// 		    if(st instanceof Highway) {

// 			if(st != strt){

// 			    next = (Highway)st;

// 			    break;
// 			}	 
// 		    }
// 		}

		if(next != null) {

// 		    if(!((Highway)strt).isHighwayEmpty(0)){
// 			next.enterHighway(((Highway)strt).highwayDequeue(0), 0);

// 			System.out.println("sent from " + strt + " to other highway" + next);
// 		    }
		    if(!((Highway)strt).isHighwayEmpty(1)){
			next.enterHighway(((Highway)strt).highwayDequeue(1), 1);			


		    }
		    
		} else {

// 		    if(!((Highway)strt).isHighwayEmpty(0)){
// 			((Highway)strt).highwayDequeue(0);
// 			System.out.println(strt);
// 			for(int z = 0; z < streetIndex.length; z++) {
// 			    System.out.print(streetIndex[z] + "->");
// 			}
// 			System.out.println("");
// 			System.out.println("Taken Off");

// 		    }
		    if(!((Highway)strt).isHighwayEmpty(1)){
			((Highway)strt).highwayDequeue(1);

		    }
		}
	    }
	}
    }// end for loop




}
