/**
 * A class providing basic street functionality
 * @author David M. Smith
 */
 
import java.awt.*;
import java.lang.*;

public class Intersection extends TrafficObject {

  protected int[] streetIndex;
  protected int[] directionIndex;
  public Coord position;

  /**
   * Construct a new intersection 
   * @param stIndex an int array listing the street indices 
   *                             terminating at this intersection 
   * @param dirIndex an int array specifying the direction of each street
   * @param p the map coordinates of the intersection
   */
  public Intersection(int[] stIndex, int[] dirIndex, Coord p ) {
    int i;
    Street st;

    streetIndex = stIndex;
    directionIndex = dirIndex;
    position = p;
    //		if( Global.DEBUG) {
    //		 	Global.debug.println( "Intersection at " + p );
    //			Global.debug.flush();
    //		}
    for( i = 0; i < streetIndex.length; i++ ) {
      st = (Street) Global.streets.find( streetIndex[i] );
      st.locate( p, directionIndex[i] );
    }
  }
	
  /**
   * draw the intersection
   * @param g the graphics environment
   */
  public void draw(Graphics g) {
    Coord here = position.screenScale();
    g.setColor(Color.lightGray);
    g.fillOval( (int) (here.x - 2*Global.STREET_WIDTH),
		(int) (here.y - 2*Global.STREET_WIDTH),
		(int) (4 * Global.STREET_WIDTH),
		(int) (4 * Global.STREET_WIDTH) );
  }


  /**
   * update the intersection
   * @param x a dummy parameter for compatibility
   */
  public void update(double x) {
    //		if( Global.DEBUG)
    //			Global.debug.println("Update " + toString());

    int index;
    Street strt;
    int direction;
    Street outStreet;
    int outDir;
    int i;
    int which;
    int choice;
    Vehicle v;
    /*
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      ;;
      ;;  this is the basic free flowing traffic control logic
      ;;  it works fine as-is.  For other flow control methods,
      ;;  derive a child class from this and change the logic accordingly
      ;; 
      ;;  4 way stop might cycle through the streets, taking only the 
      ;;    first available car from one street
      ;;
      ;;  traffic lights might permit cars from odd or even indexed streets 
      ;;    switching to the other state after some period of time
    */
    for( i = 0; i < streetIndex.length; i++ ) {
      index = streetIndex[i];
      strt = (Street) Global.streets.find(index);
      direction = directionIndex[i];
      //  if(Global.DEBUG)
      //	Global.debug.print("<" + strt + ">");
      if( !strt.isEmpty(direction) )  {
	v = strt.peek(direction);
	// pick the outgoing street
	v.turn(this, strt, direction);
	outStreet = v.getNextStreet();
	outDir = v.getNextDirection();
	if( outStreet == null ) {
	  strt.dequeue(direction);
	} else {
	  if( !outStreet.isFull(outDir, v) ) {
	    try {
	      outStreet.enqueue( strt.dequeue(direction), outDir );
	    } catch( Exception e) { 
	      System.out.print("Error " + e + " dequeuing" ); 
	      Global.debug.println("Error " + e + " dequeuing" );
	    } 
	  }
	}
      }
    }
  }

  /**
   * document the contents of the intersection
   * @return the string describing the intersection
   */
  public String toString() {
    String res = "Intersection at " + position + "{ "; 
    int i;

    for(i = 0; i < streetIndex.length; i++ ) {
      res = res + "[" + streetIndex[i] + "," 
	+ directionIndex[i] + "]";
    }
    res = res + " }";
    return res;
  }

}


