/**
 * A class providing the functionality of a traversible queue
 * @author David M. Smith
 */
 
import java.lang.*;
import java.awt.*;

public class StreetList extends TravQueue { 


  /**
   * call the display method for item on the street list
   * @param dt the time difference in seconds
   */
  public void display() {
    ListNode here = head;
    Street st;

    Global.debug.println("Street List");
    while( here != null ) {
      st = (Street) here.data;
      st.display();
      here = here.next;
    }
  }


  /**
   * call the update method for each item on the street list
   * @param dt the time difference in seconds
   */
  public void update(double dt) {
    ListNode here = head;
    Street st;

    while( here != null ) {
      st = (Street) here.data;
      st.update(dt);
      here = here.next;
    }
  }



  /** 
   * insert vehicles at one street end
   * @param str the index of the street
   * @param dir the street direction
   */
  public static void insertVehicles(int str, int dir) {
    double x = Math.random();
    Street st = (Street) Global.streets.find( str );
    Coord loc;

    loc = st.location[1-dir].sum( st.direction[1-dir].normal().scale( 1/Global.SCALE ) );
    x = Global.MAX_INSERT * x;
    while( x > 0.5 ) {
      x--;
      st.enqueue( Vehicle.randomVehicle(), dir );
    }
  }



  /**
   * find the nth item in the queue
   * @param index the index of the item to find
   * @return the nth Object on the queue, null if n is too big
   */
  public Object find(int index) {
    ListNode here = head;
    int i = 0;
    Object res = null;

    while( (here != null) && (i < index) ) {
      here = here.next;
      i++;
    }
    if( here != null ) res = here.data;
    return res;
  }



}


