/**
 * A class providing the graphics shell for traffic modeling
 * @author David M. Smith
 */
 
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;

public class Traffic extends JFrame {

  private Map myMap;  
  private Thread mapThread;

  
  /**
   *   Traffic constructor
   */
  public Traffic (boolean isLarge) {
    
    EventHandler eh;

    try { 
      Global.debug = new PrintStream( new FileOutputStream( "debug.log" ) ); 
    } 
    catch( IOException e) { 
      System.out.print("Error " + e + " opening debug.log" ); 
    } 
    Global.debug.println( "Opened log file" );
    Global.debug.flush();

    setTitle("Traffic Map");
        
    addWindowListener( new WindowAdapter () {
         
	public void windowClosing(WindowEvent e) {
         
	  myMap.stop();
	  ( (JFrame) e.getSource() ).dispose();
	  System.exit(0);
	}
      });
    
//    setSize (1000,700);
    setSize (780,560);
    setLocation (10,10);
    JPanel panel = new JPanel();
    panel.setLayout(new FlowLayout());
    JButton startB = new JButton( "Start" );
    eh = new EventHandler (this, startB);
    startB.addMouseListener(eh);
    panel.add(startB);
    JButton addB = new JButton( "Go" );
    eh = new EventHandler (this, addB);
    addB.addMouseListener(eh);
    panel.add(addB);
    JButton stepB = new JButton( "Step" );
    eh = new EventHandler (this, stepB);
    stepB.addMouseListener(eh);
    panel.add(stepB);
    JButton stopB = new JButton( "Stop" );
    eh = new EventHandler (this, stopB);
    stopB.addMouseListener(eh);
    panel.add(stopB);
    Container cp = getContentPane();
    cp.setLayout(new BorderLayout());
    myMap = new Map( new Dimension(700, 500), isLarge );
    cp.add( myMap, BorderLayout.CENTER );
    cp.add( panel, BorderLayout.SOUTH );
    show();
  }


  /**
   *   Event handler for mouse clicks
   */
  class EventHandler implements MouseListener {  
    Traffic theFrame; 
    JButton button;
    
    public EventHandler (Traffic theFrame, JButton button) {
      this.theFrame = theFrame;
      this.button  = button;
    }

    public void mouseClicked(MouseEvent e) {
      String com = button.getText();
      if (com.equals("Start")) {
	mapThread = new Thread(myMap);
	myMap.isRunning = true;
	mapThread.start();
      } else if (com.equals("Go")) {
			if( !Global.Quake ) {
				Global.Quake = true;
				System.out.println("Earthquake happened!");
			} else if( !Global.isstall ) {
				Global.isstall = true;
				System.out.println("Vehicle stalled");
			} else Vehicle.bumpSmarts();
      } else if (com.equals("Step")) {
	myMap.stop();
	mapThread = new Thread(myMap);
	mapThread.start();
      } else if (com.equals("Stop")) {
	myMap.stop();
      } else {
	Global.debug.println( "Erroneous Action" );
      }
    }
    public void mousePressed(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e)  {}
    public void mouseReleased(MouseEvent e){}
    
  } 

  public static void main(String[] s) {
    String input = JOptionPane.showInputDialog("Large model <L> or small <S>?");

    boolean choice = (input.charAt(0) == 'L') || (input.charAt(0) == 'l');

    Traffic t = new Traffic(choice);
  }

}  // Traffic




