/**
 * A class providing the functionality of a traversible queue
 * @author David M. Smith
 */
 
import java.lang.*;
import java.awt.*;

public class TravQueue extends Queue { 

  ListNode nextNode = head;
  /**
   * call the draw method each item on the queue
   * @param g the graphics environment
   */

  public void draw(Graphics g) {
    ListNode here = head;

    while( here != null ) {
      ((TrafficObject) here.data).draw(g);
      here = here.next;
    }
  }


  /**
   * call the update method each item on the queue
   * @param distance the distance in map coordinates to travel
   */

  public void update(double distance) {
    ListNode here = head;

    while( here != null ) {
      ((TrafficObject) here.data).update(distance);
      here = here.next;
    }
  }


  /**
   * separate each car on the queue
   *  starting at the given position, space the cars in the queue according to the
   *  speed they are traveling.  If they are too close to the car in front,
   *  move them back to the right distance.  Return the position of the back of the
   *  queue.
   * @param speed the spped on the road segment
   * @param dir a Coord specifyingthe queue direction
   * @param start a Coord specifying the head of the queue
   * @return the Coord that is the new end of the queue
   */

  public Coord separate(int speed, Coord dir, Coord start ) {
    Coord back = start;
    Vehicle v;
    ListNode here;

    if( !isEmpty() ) {
      here = head;
      while( here != null ) {
	v = (Vehicle) here.data;
	back = v.shuffle( back, speed, dir );
	here = here.next;
      }
    }
    return back;
  }

  /**
   * reset an iterator
   */
  public Object reset() {
    nextNode = head;
    return getNext();
  }

  /**
   * next node
   */
  public Object getNext() {
    Object ans;

    if( nextNode == null ) return null;
    else {
      ans = nextNode.data;
      nextNode = nextNode.next;
      return ans;
    }
  }
}


